#--
#-- SUB: performCompleteRefresh
#-- Performs a COMPLETE REFRESH on a snapshot
#--
sub performCompleteRefresh {
	#-- Function parameters
	my ($dbh_local, $dbh_remote, $snapshot) = @_;

	#-- Local variables
	my $sql;
	my $sth;
	my $sth_local;
	my $recs;
	my $errors;
	my $snapshotname;
	my $targetColumnList;

	#--Fetches the remote database query
	$sql = $snapshot->{'query'};
	elog NOTICE, "sql is \n$sql" if $main::DEBUG==1;

	$sth = $dbh_remote->prepare($sql);
	if ($dbh_remote->err) {
		elog ERROR, "Could not fetch remote query: $sql [".$dbh_remote->errstr."]";
	}
	$sth->execute;
	#-- Parameterized query placeholders (question marks)
	my $qms = '?,' x $sth->{NUM_OF_FIELDS}; 
	chop($qms);

	$snapshotname = ($snapshot->{'pbt_table'} eq '') ? $snapshot->{'snapshotname'} : $snapshot->{'pbt_table'};
	$targetColumnList = join(',', getColumnList($sth));
	if ($snapshot->{'pbt_table'} eq '') {
		#--Truncates (empty) the local SNAPSHOT
		$sql = <<SQL;
TRUNCATE $snapshot->{'schemaname'}.$snapshotname
SQL
		$dbh_local->do($sql);
		if (! $dbh_local->err) {
			elog NOTICE, "Snapshot truncated" if $main::DEBUG==1;
		} else {
			elog ERROR, "Could not truncate snapshot: $sql [".$dbh_local->errstr."]";
		}
	} else {
		setTriggerStatus($dbh_local, $snapshot->{'schemaname'}, $snapshotname, "${snapshotname}_pbt\$_trg", FALSE);
		$targetColumnList = "pbt\$,$targetColumnList";
		$qms = "$snapshot->{'snapid'},$qms";
		#-- Deletes all rows where pbt$ eq $snapid
		$sql = <<SQL;
DELETE FROM $snapshot->{'schemaname'}.$snapshotname
WHERE pbt\$ = ?
SQL
		my $sth_aux = $dbh_local->prepare($sql);
		$sth_aux->execute(($snapshot->{'snapid'}));
		if (! $sth_aux->err) {
			elog NOTICE, "Snapshot cleaned." if $main::DEBUG==1;
		} else {
			elog ERROR, "Could not clean snapshot: $sql [".$sth_aux->errstr."]";
		}
	}
	$dbh_local->commit;

	#-- Fill the SNAPSHOT with the remote database query results
	
		#-- Prepare the INSERT query that will be executed for each returned query
		$sql = <<SQL;
INSERT INTO $snapshot->{'schemaname'}.$snapshotname($targetColumnList)
VALUES ($qms)
SQL
		elog NOTICE, $sql if $main::DEBUG==1;
		$sth_local = $dbh_local->prepare($sql);

		#-- Set the type of target row parameters to the same type of source row columns
		my @types = getSthColumnTypes($sth);
		for (my $count=0; $count < $sth->{NUM_OF_FIELDS}; $count++) {
			$sth_local->bind_param($count + 1, undef, @types[$count]);
		}

		#-- Loop through all fetched records and insert them one at a time into the SNAPSHOT
		$recs = $errors = 0;
		while(my @row = $sth->fetchrow_array) {
			$sth_local->execute(@row);
			if ($sth_local->err) {
				if ($errors eq 0) {
					elog ERROR, $sth_local->errstr;
				}
				++$errors;
			}
			++$recs;
			if (($recs % 1000) == 0) {
				$dbh_local->commit;
				elog NOTICE, "Commit. Record #$recs" if $main::DEBUG==1;
			}
		}
		if ($sth->err) {
			elog ERROR, $sth->errstr;
		}
		$dbh_local->commit;
		$sth_local->finish;

		$sth->finish;

		elog NOTICE, "All $recs records processed. $errors errors." if $main::DEBUG==1;
	if ($snapshot->{'pbt_table'} ne '') {
		setTriggerStatus($dbh_local, $snapshot->{'schemaname'}, $snapshotname, "${snapshotname}_pbt\$_trg", TRUE);
	}
	#-- Returns the number of records inserted
	return $recs-$errors;
}
