#--
#-- SUB: performFastRefresh
#-- Performs a FAST REFRESH on a snapshot
#--
sub performFastRefresh {
	#-- Function parameters
	my ($dbh_local, $dbh_remote, $snapshot, $masterschema, $mastername, $lastRefresh) = @_;

	#-- Local variables
	my $sth_local;
	my $sth_remote;
	my $masterKeys = getSnapshotLogColumns($dbh_remote, $masterschema, $mastername);
	my $qms;
	my $recs;
	my $errors;
	my $sql;
	my $where;
	my @types;
	my ($logKeys, $arrLogKeys) = getLogKeys ($masterKeys);
	my @keys = ($masterKeys, $logKeys, @$arrLogKeys);
	my $snapshotLogName = getSnapshotLogName($dbh_remote, $masterschema, $mastername);
	my $snapshotname;
	my $targetColumnList;

	#-- Remove old records from the snapshot
	my $deleted = deleteOldSnapshotRecords($dbh_local, $dbh_remote, $snapshot, $masterschema, $mastername, $lastRefresh, @keys);

	my @arrMasterKeys = getSourceKeysFromQuery($snapshot->{'query'}, split(',', $masterKeys));
	my $filter = '';
	for (my $i = 0; $i < @arrMasterKeys; ++$i) {
		$filter .= ' AND source.'.@arrMasterKeys[$i].'=keys.'.@$arrLogKeys[$i];
	}
	$filter = substr($filter, 5);
	my $logFilter = getInsertedUpdatedLogRecordsFilter($dbh_remote, $lastRefresh);
	my $source = $snapshot->{'query'};
	$sql = <<SQL;
SELECT source.*
FROM ($source) source,
	(SELECT $logKeys
	FROM $masterschema.$snapshotLogName
	WHERE $logFilter) keys
WHERE $filter
SQL
	elog NOTICE, "Filter SQL is: $sql" if $main::DEBUG==1;
	elog NOTICE, "LastRefresh is: $lastRefresh" if $main::DEBUG==1;
	$sth_remote = $dbh_remote->prepare($sql);
	if ($dbh_remote->err) {
		elog ERROR, "Could not apply filter rules to original query: $sql [".$dbh_remote->errstr."]";
	}

	#-- Fill it with the result query
	$sth_remote->execute();
	if ($dbh_remote->err) {
		elog ERROR, "Could not retrieve records to insert: $sql [".$dbh_remote->errstr."]";
	}

	$qms = '?,' x $sth_remote->{NUM_OF_FIELDS};
	chop($qms);

	if ($snapshot->{'pbt_table'} ne '') {
		$qms = "$snapshot->{'snapid'},$qms";
	}

	$snapshotname = ($snapshot->{'pbt_table'} eq '') ? $snapshot->{'snapshotname'} : $snapshot->{'pbt_table'};
	$targetColumnList = join(',', getColumnList($sth_remote));

	if ($snapshot->{'pbt_table'} ne '') {
		$targetColumnList = "pbt\$,$targetColumnList";
	}
	$sql = <<SQL;
INSERT INTO $snapshot->{'schemaname'}.$snapshotname($targetColumnList)
VALUES ($qms)
SQL
	$sth_local = $dbh_local->prepare($sql);
	if ($dbh_local->err) {
		elog ERROR, "Could not add new records into snapshot: $sql [".$dbh_local->errstr."]";
	}

	@types = getSthColumnTypes($sth_remote);
	for (my $count=0; $count < $sth_remote->{NUM_OF_FIELDS}; $count++) {
		$sth_local->bind_param($count + 1, undef, @types[$count]);
	}

	elog NOTICE, 'Fetching records.' if $main::DEBUG==1;
	#-- Loop through all fetched records and insert them one at a time into the SNAPSHOT
	$recs = 0;
	$errors = 0;
	setTriggerStatus($dbh_local, $snapshot->{'schemaname'}, $snapshotname, "${snapshotname}_pbt\$_trg", FALSE);
	while(my @row = $sth_remote->fetchrow_array) {
		$sth_local->execute(@row);
		if ($sth_local->err) {
			++$errors;
		}
		++$recs;
		if (($recs % 1000) == 0) {
			$dbh_local->commit;
			elog NOTICE, "Inserted. Record #$recs" if $main::DEBUG==1;
		}
	}
	setTriggerStatus($dbh_local, $snapshot->{'schemaname'}, $snapshotname, "${snapshotname}_pbt\$_trg", TRUE);
	if (! $sth_remote->err) {
		$recs = $recs - $errors;
		elog NOTICE, "Inserted $recs modified records" if $main::DEBUG==1;
		if ($errors > 0) {
			elog ERROR, "$errors errors on insertion process" if $main::DEBUG==1;
		}
	} else {
		elog ERROR, "Could insert modified snapshot records: $sql [".$sth_remote->errstr."]";
	}

	$dbh_local->commit;
	$sth_local->finish;

	return $recs;
}

