#--
#-- SUB: retrieveMasterForSnapshot
#-- Retrieve the MASTER tables for a SNAPSHOT
#--
sub retrieveMasterForSnapshot {
	#-- Function parameters
	my ($dbh, $snapshot) = @_;

	#-- Local variables
	my $masterschema;
	my $mastername;
	my $sql;
	my $sth;
	my $relatedCount;
	my $tablename;
	my @related=();
	my @tables=();

	$tablename = $snapshot->{'query'};
	@related = $tablename =~ m/(from|join) +((([0-9a-z_\$]+|".+")\.)?([0-9a-z_\$]+|".+")( *, *(([0-9a-z_\$]+|".+")\.)?([0-9a-z_\$]+|".+"))*)/gi;

	for (my $i = 1; $i < @related; $i += 9) {
		push(@tables, @related[$i]);
	}

	if (@tables eq 0) {
		elog ERROR, 'Origin TABLES not found !';
	}

	if (@tables eq 1) {
		my $tablename = @tables[0];
		elog NOTICE, "TABLE=[$tablename]" if $main::DEBUG==1;
		($masterschema, $mastername) = split(/\./, $tablename);
		elog NOTICE, "MASTERSCHEMA=[$masterschema] MASTERNAME=[$mastername]" if $main::DEBUG==1;
		if ($mastername eq undef) {
			#-- No Schema
			elog NOTICE, 'Sorry, due to the complexity of your query, I could not guess the Schema name of your tables, so I will try a COMPLETE REFRESH instead.';
			$mastername = $masterschema;
			$snapshot->{'kind'} = 'C';
			$masterschema = undef;
			
		}
		return ($masterschema, $mastername);
	}

	if (($snapshot->{'kind'} eq 'F') || ($snapshot->{'kind'} eq 'R')) {
		elog NOTICE, "Refresh FAST only work with one-table only queries. Falling back to COMPLETE REFRESH. TABLES=[" . join(',', @tables) . ']';
		$snapshot->{'kind'} = 'C';
	}
	return (undef, undef);
}
