#--
#-- SUB: setTriggerStatus
#-- Enable/Disable a trigger
#--
sub setTriggerStatus {
	#-- Function parameters
	my ($dbh, $schemaname, $tablename, $triggername, $status) = @_;

	#-- Local variables
	my $sql;
	my $sth;
	my $tgenabled = ($status == TRUE) ? 'TRUE' : 'FALSE';

	$sql = <<SQL;
SELECT c.oid
FROM pg_class c
	LEFT JOIN pg_namespace n ON n.oid = c.relnamespace
WHERE n.nspname=? 
	AND c.relname=?
SQL
	my ($tableid) = sqlLookup($dbh, $sql, ($schemaname, $tablename));
	if ("$tableid" eq "") {
		elog ERROR, "Could not retrieve TABLEID for table $schemaname.$tablename";
	}

	elog NOTICE, "Disabling trigger for table ID=$tableid NAME=$triggername" if $main::DEBUG==1;
	$sql = <<SQL;
UPDATE pg_trigger 
SET tgenabled = $tgenabled 
WHERE tgrelid=? 
	AND tgname=?
SQL
	elog NOTICE, $sql if $main::DEBUG==1;
	$sth = $dbh->prepare($sql);
	$sth->execute(($tableid, $triggername));

	#-- Dummy update to refresh new trigger status
	if ($sth->err) {
		elog ERROR, $sth->errstr;
	}
	$sql = <<SQL;
UPDATE pg_class
SET relname=relname
WHERE oid=?
SQL
	$sth = $dbh->prepare($sql);
	$sth->execute(($tableid));
	if ($sth->err) {
		elog ERROR, $sth->errstr;
	}
}
